function rainSTORM_bridgeTracker(reviewer,figNewReconHandle)
% 	Summary:	
%	Version:	0.2
%	Note:		http://devopsreactions.tumblr.com/post/75139222312/looking-at-someone-elses-code
%	Package:	rainSTORM v3.1+    (based on Box Tracker)
%	Developer:	gajdost
%	Name:		Gajdos Tamás
%	Company:	University of Szeged

% FUNCTION
%   Selects quality-controlled localizations within a user-defined box
%   Plots the data as a scatterplot, colour coded by sigma_x/sigma_y
%   May be used to inspect data for bridge artifact
%
% USER NOTES
% 0. This tool identifies localizations within a user-defined rectangle.
% 1. Apply this tool to the most recent super-res reconstruction Figure
%    It searches for reviewed Posits - i.e. accepted Localisations -
%    which are only retained for one Figure at a time
%    To avoid confusion, I recommend closing all figure windows, and then
%    plotting a Reviewed reconstruction to work on with this tool - EJR.
% 2. The tool allow you to select one rectangular region within the
%    Super-resolution reconstruction.
% 3. All localisations within this region are identified, and stored as
%    bridgedPosits and associated bridgedParams
% 4. Based on the box tracker code.
%% Part 0: Flow Control and Input Arguments
if verLessThan('matlab','8.4.0')
    reviewedParams = reviewer.results.reviewedSupResParams;
else
    if (strfind(figNewReconHandle.Name,'rejSupImage'))
        reviewedParams = reviewer.results.rejectedSupResParams;
    else
        reviewedParams = reviewer.results.reviewedSupResParams;
    end
end

reviewedPosits = [[reviewedParams.x_coord]' [reviewedParams.y_coord]'];
reconstructionScaleFactor = reviewer.results.reconstructionScaleFactor;

%%Identify user-boxed localisations
rect = getrect(figNewReconHandle);
% Above: user must use a UI to specify a region of the reconstuction.
bridgeCols = [rect(1), rect(1) + rect(3) ] ./ reconstructionScaleFactor;
bridgeRows = [rect(2), rect(2) + rect(4) ] ./ reconstructionScaleFactor;

selectedRows = ( (reviewedPosits(:,1) > bridgeRows(1)) & ...
    (reviewedPosits(:,1) < bridgeRows(2)) & ...
    (reviewedPosits(:,2) > bridgeCols(1)) & ...
    (reviewedPosits(:,2) < bridgeCols(2)) );

bridgedParams = reviewedParams( selectedRows )';
bridgedPosits = reviewedPosits( selectedRows, :);
abs_sig_xy = abs([ bridgedParams.sig_x ]' ./ [ bridgedParams.sig_y ]' -1);
x_std_local = [ bridgedParams.x_std ];
y_std_local = [ bridgedParams.y_std ];

%% Display
% Note that Y-positions are plotted as negative row positions,
% to match image matrix orientation.

% Plot graphs
figSize = [1,100,1120,840];
figHistsHandle = figure(...
    'Name','Bridge Tracking',...
    'NumberTitle','off',...
    'OuterPosition', figSize ...
    );

% First Subplot:
subplot(2,2,1) % Use this if screen shape spoils the method below

h=scatter(bridgedPosits(:,2), -bridgedPosits(:,1), 14, abs_sig_xy, 'x');
set(gca,'FontName','Times New Roman','Fontsize',12)
title('Bridge Positions (Colour = | sig_x/sig_y -1 |)', 'fontSize', 12, 'fontWeight', 'bold');
xlabel('CCD Col Number', 'fontSize', 12, 'fontWeight', 'bold');
ylabel('CCD Row Number (Negative for image alignment)', 'fontSize', 12, 'fontWeight', 'bold');
colorbar
caxis([0,0.2])
set(gca, 'fontSize', 12, 'fontWeight', 'bold');
axis equal % Set axis increments to be of equal size; prevent stretching

%% Histogram for first subplot;
subplot(2,2,2)

bins_abs_sig_xy = 0.0125:0.025:1;
hist(abs_sig_xy,bins_abs_sig_xy);
title('Fit ellipticity histogram');
xlabel('abs(sig_x/sig_y - 1)')
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');

%% Third supblot
subplot(2,2,3)

scatter(bridgedPosits(:,2), -bridgedPosits(:,1), 14, x_std_local, 'x')
set(gca,'FontName','Times New Roman','Fontsize',12)
title('Bridge Positions (Thompson X STD)', 'fontSize', 12, 'fontWeight', 'bold');
xlabel('CCD Col Number', 'fontSize', 12, 'fontWeight', 'bold');
ylabel('CCD Row Number (Negative for image alignment)', 'fontSize', 12, 'fontWeight', 'bold');
colorbar
caxis([0,25])
set(gca, 'fontSize', 12, 'fontWeight', 'bold');
axis equal % Set axis increments to be of equal size; prevent stretching

%% Histogram for third subplot
% subplot(2,2,4)
% hist(residue_local);
% title('Residue histogram');
% xlabel('Residue')
% set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');

%% Fourth subplot
subplot(2,2,4);
scatter(bridgedPosits(:,2), -bridgedPosits(:,1), 14, y_std_local, 'x')
set(gca,'FontName','Times New Roman','Fontsize',12)
title('Bridge Positions (Thompson Y STD)', 'fontSize', 12, 'fontWeight', 'bold');
xlabel('CCD Col Number', 'fontSize', 12, 'fontWeight', 'bold');
ylabel('CCD Row Number (Negative for image alignment)', 'fontSize', 12, 'fontWeight', 'bold');
colorbar
caxis([0,25])
set(gca, 'fontSize', 12, 'fontWeight', 'bold');
axis equal % Set axis increments to be of equal size; prevent stretching

