function settings = rainSTORM_default_settings_loc_filters(varargin)

if isempty(varargin)
    settings.names = {...
        'rainSTORM algo 1-3';...
        'Box Tracking';...
        'Test Filter'};
    settings.fhandle = {...
        @(SupResParams,settings,flagDebug)rainSTORM_loc_filter(SupResParams,settings,flagDebug);...
        @(SupResParams,settings,flagDebug)rainSTORM_loc_filter_box_tracking(SupResParams,settings,flagDebug);...
        @(SupResParams,settings,flagDebug)rainSTORM_loc_filter_test_algo(SupResParams,settings,flagDebug)};
    return
end

locfilter_name = varargin{1};
switch locfilter_name
    case 'rainSTORM algo 1-3'
        settings = gen_struct_1;
    case 'Box Tracking'
        settings = gen_struct_2;
    case 'Test Filter'
        settings = gen_struct_3;
    otherwise
        error(['Unknown Recon Algo Name: ', locfilter_name])
end

end

function settings = gen_struct_1()
settings=[];
settings.frame_idx = struct(...
    'val',[0 Inf],...
    'inverted',0,...
    'text','Frame Index',...
    'tooltip','The range of frames which shall be included. [frame index]',...
    'type','number');
settings.x_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','X coordinate',...
    'tooltip','The range of localization coordinates in the x direction. [pix]',...
    'type','number');
settings.y_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','Y coordinate',...
    'tooltip','The range of localization coordinates in the y direction. [pix]',...
    'type','number');
settings.z_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','Z coordinate',...
    'tooltip','The range of localization coordinates in the z direction. [nm]',...
    'type','number');
settings.x_std = struct(...
    'val',[0 45],...
    'inverted',0,...
    'text','Std X',...
    'tooltip','The localization uncertainty in the x direction. [pix]',...
    'type','number');
settings.y_std = struct(...
    'val',[0 45],...
    'inverted',0,...
    'text','Std Y',...
    'tooltip','The localization uncertainty in the y direction. [pix]',...
    'type','number');
settings.I = struct(...
    'val',[0 Inf],...
    'inverted',0,...
    'text','I parameter',...
    'tooltip','The range of I parameter. [min max]',...
    'type','number');
settings.sig_x = struct(...
    'val',[0.7 4],...
    'inverted',0,...
    'text','Sigma X',...
    'tooltip','The range of localization uncertainty in the x direction. [pix]',...
    'type','number');
settings.sig_y = struct(...
    'val',[0.7 4],...
    'inverted',0,...
    'text','Sigma Y',...
    'tooltip','The range of localization uncertainty in the y direction. [pix]',...
    'type','number');
settings.avg_brightness = struct(...
    'val',[0 Inf],...
    'inverted',0,...
    'text','Average Brightness',...
    'tooltip','The range of brightness in the ROI area of localization. [counts]',...
    'type','number');
settings.res = struct(...
    'val',[0 0.5],...
    'inverted',0,...
    'text','Residue',...
    'tooltip','The reside of the localization. [-]',...
    'type','number');
settings.res_Row = struct(...
    'val',[0 1],...
    'inverted',0,...
    'text','Residue Row',...
    'tooltip','The reside of the localization in the row direction. [-]',...
    'type','number');
settings.res_Col = struct(...
    'val',[0 1],...
    'inverted',0,...
    'text','Residue Col',...
    'tooltip','The reside of the localization in the column direction. [-]',...
    'type','number');
settings.roi_min = struct(...
    'val',[0 Inf],...
    'inverted',0,...
    'text','ROI Min',...
    'tooltip','The minimum pixel value in the ROI area. [counts]',...
    'type','number');
settings.Sum_signal = struct(...
    'val',[0 Inf],...
    'inverted',0,...
    'text','Sum Signal',...
    'tooltip','The sum of the pixel values in the ROI area. [counts]',...
    'type','number');

end


function settings = gen_struct_2()

settings=[];
settings.x_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','X coordinate',...
    'tooltip','The range of localization coordinates in the x direction. [pix]',...
    'type','number');
settings.y_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','Y coordinate',...
    'tooltip','The range of localization coordinates in the y direction. [pix]',...
    'type','number');

end


function settings = gen_struct_3()

settings=[];
for idx_param = 1:11
settings.(['Test',num2str(idx_param,'%2.2i')]) = struct(...
    'val',idx_param,...
    'text',['Test',num2str(idx_param,'%2.2i'),' [-]'],...
    'tooltip',['Test ',num2str(idx_param),' this value. [-]'],...
    'type','number');
end

end
