function results = rainSTORM_image_reviewer(SupResParams,settings)
% flagDebug: A development feature, not used in every-day-dSTORMin'
% If the filer debugging is enabled:
% - pass logical and create the filter histograms plot
% - create the rejected Localizations struct, which is a negate.
% - apply drift correction to rejected also
% Otherwise:
% - empty struct and do nothing with it.
% <<EOL>>

tic
% get the available filter and reconstruction algo names and handles
algo_names_and_handles_loc_filter = rainSTORM_default_settings_loc_filters;
algo_names_and_handles_recon = rainSTORM_default_settings_recon_algo;

% filter the localization data
qualityApprovedLocs = true(size(SupResParams));
for idx_filter = 1:length(settings.filters)
    loc_filter_handle = algo_names_and_handles_loc_filter.fhandle{...
        (strcmp(algo_names_and_handles_loc_filter.names,settings.filters{idx_filter}.filter_name)==1)};
    [accepted_localizations, filterOutputHists] = loc_filter_handle(SupResParams,settings.filters{idx_filter}.settings,settings.flagDebug);
    qualityApprovedLocs = qualityApprovedLocs & accepted_localizations;
end
reviewedSupResParams = SupResParams(qualityApprovedLocs);
if (settings.flagDebug)
    rejectedSupResParams = SupResParams(~qualityApprovedLocs);
else
    rejectedSupResParams = struct([]);
end

% Substract Drift if it is specified.
if isfield(settings, 'undrift') && ~isempty(settings.undrift)
    reviewedSupResParams = rainSTORM_undrift_denoised(reviewedSupResParams,settings);
    if (settings.flagDebug)
        rejectedSupResParams = rainSTORM_undrift_denoised(rejectedSupResParams,settings);
    end
end

% Substract Automatic Drift if it is specified.
if isfield(settings, 'auto_undrift') && ~isempty(settings.auto_undrift)
    fprintf( 1, 'Applying "automatic" drift correction!\n' );
%     dx_c = num2cell( settings.auto_undrift.params.dx' );
%     dy_c = num2cell( settings.auto_undrift.params.dy' );
%     size(settings.auto_undrift.params.dx)
%     size(deal(dx_c{:}))
    frame_idx = [reviewedSupResParams.frame_idx];
    new_x_coords_c = num2cell([reviewedSupResParams.x_coord] - settings.auto_undrift.params.dx(frame_idx));
    new_y_coords_c = num2cell([reviewedSupResParams.y_coord] - settings.auto_undrift.params.dy(frame_idx));
    [reviewedSupResParams(:).x_coord] = deal( new_x_coords_c{:} );
    [reviewedSupResParams(:).y_coord] = deal( new_y_coords_c{:} );
% COMMENT: This is an unresolved part.    
%    if (settings.flagDebug)
%        newrej_x_coords_c = num2cell([rejectedSupResParams.x_coord] - settings.auto_undrift.params.dx(frame_idx));
%        newrej_y_coords_c = num2cell([rejectedSupResParams.y_coord] - settings.auto_undrift.params.dy(frame_idx));
%        [rejectedSupResParams(:).x_coord] = deal( newrej_x_coords_c{:} );
%        [rejectedSupResParams(:).y_coord] = deal( newrej_y_coords_c{:} );
%    end
end

% Generate the super resolved image
recon_algo_handle = algo_names_and_handles_recon.fhandle{...
        strcmp(algo_names_and_handles_recon.names,settings.recon_algo_name)==1};
[SupResIm, recon_algo_outputs] = recon_algo_handle(reviewedSupResParams,settings.recon_algo_settings);
% Rejected Image, not so super, but an ugly one
if (settings.flagDebug)
    [rejSupIm, ~] = recon_algo_handle(rejectedSupResParams,settings.recon_algo_settings);
% ... and store rejected coordinates
    rejSupCo.x_coord = [([rejectedSupResParams.x_coord]' * settings.recon_algo_settings.linMag.val)]';
    rejSupCo.y_coord = [([rejectedSupResParams.y_coord]' * settings.recon_algo_settings.linMag.val)]';
end

% Calculate some metrics for later useage
reviewedDeltaX = [[reviewedSupResParams.x_std]' [reviewedSupResParams.y_std]'];

meanRevDeltaX = mean(reviewedDeltaX); % Mean precision in reconstruction
stdRevDeltaX  = std(reviewedDeltaX);  % Std Dev of precisions- poor metric?
SparrowThompsonLimit = 2*sqrt(mean(reviewedDeltaX.^2)); % Approx. 'resolution'

numberOfFrames  = max([SupResParams.frame_idx]);
numberAcceptedPerFrame = zeros(1,numberOfFrames, 'uint16');
numberCandidatesPerFrame = zeros(1,numberOfFrames, 'uint16');
% FIXME: This needs some rework I think.
for lpLoc = 1:length(reviewedSupResParams)
  numberAcceptedPerFrame(reviewedSupResParams(lpLoc).frame_idx) = ...
      numberAcceptedPerFrame(reviewedSupResParams(lpLoc).frame_idx)+1;
end
for lpLoc = 1:length(SupResParams)
  numberCandidatesPerFrame(SupResParams(lpLoc).frame_idx) = ...
      numberCandidatesPerFrame(SupResParams(lpLoc).frame_idx)+1;
end
numberRejectedPerFrame = numberCandidatesPerFrame - numberAcceptedPerFrame;

% store the output data
results.SupResIm = SupResIm;
if (settings.flagDebug)
    results.rejSupIm = rejSupIm;
    results.rejSupCo = rejSupCo;
end
results.reviewedSupResParams = reviewedSupResParams;
results.rejectedSupResParams = rejectedSupResParams;
results.SparrowThompsonLimit = SparrowThompsonLimit;
results.densestPoint = max(SupResIm(:));
results.meanRevDeltaX = meanRevDeltaX;
results.numberOfFits = size(SupResParams,1);
results.qualityApprovedLocs = qualityApprovedLocs;
results.numberAcceptedPerFrame = numberAcceptedPerFrame;
results.numberRejectedPerFrame = numberRejectedPerFrame;
results.stdRevDeltaX = stdRevDeltaX;
results.reconstructionScaleFactor = recon_algo_outputs.reconstructionScaleFactor;
results.recon_algo_outputs = recon_algo_outputs;
results.filterOutputHists = filterOutputHists;
toc
end